@@ -205,8 +205,7 @@ class PostCleaner(GenericPostCleaner):
         self.link_rel_escape = link_rel_escape
 
     def append_escape(self, s):
-        #self.append(XmlEntities(s))
-        self.append(s)
+        self.append(XmlEntities(s))
 
     def append_url(self, scheme, url): # scheme includes ://
         self.append('<a href="')
